class

	CHILD_DEFINITION_CHOICE


inherit

	CHILD_DEFINITION


creation

	make,
	make_from_array


feature {NONE} -- Initialization

	make (a_choices: DS_LINKED_LIST [CHILD_DEFINITION]) is
		require
			choices_not_void: a_choices /= Void
		do
			choices := a_choices
		end

	make_from_array (a_array: ARRAY [CHILD_DEFINITION]) is
		require
			array_not_void: a_array /= Void
			array_has_at_least_one_element: a_array.count >= 1
		local
			i: INTEGER
		do
			create choices.make
			from
				i := a_array.lower
			until
				i > a_array.upper
			loop
				choices.put_last (a_array.item (i))
				i:= i + 1
			end
		end


feature -- state

	choices: DS_LINKED_LIST [CHILD_DEFINITION]


feature {CHILD_DEFINITION} -- start tag transition

	is_allowed_set (state: ELEMENT_STATE;
						 next_element: DEFINED_ELEMENT): BOOLEAN is
		local
			cursor: DS_LINEAR_CURSOR [CHILD_DEFINITION]
			child_state: CHILD_STATE
		do
			debug
				print ("CHILD_DEFINITION_CHOICE.is_allowed_set%N")
			end

			-- push new state level on stack, this object becomes active
			-- we need this so `any_required_child' can return False
			create child_state.make (Current)
			state.children.put (child_state)

			-- if next_element occurs in choices, then it's ok
			from
				cursor := choices.new_cursor
				cursor.start
			until
				Result or else cursor.after
			loop
				debug
					print ("  evaluating choice.%N")
				end
				Result := cursor.item.is_allowed_set (state, next_element)
				cursor.forth
			end

			if not Result then
				state.children.remove
			end
		end

	is_allowed_stay (state: ELEMENT_STATE;
						  next_element: DEFINED_ELEMENT): BOOLEAN is
		do
			debug
				print ("CHILD_DEFINITION_CHOICE.is_allowed_stay%N")
			end

			-- we cannot stay, make sure our state is removed
			Result := False
			state.children.remove
		ensure
			stack_decrease: state.children.count = old state.children.count - 1
		end


feature {DEFINED_ELEMENT, CHILD_DEFINITION} -- end tag transition

	any_required_child (state: ELEMENT_STATE): BOOLEAN is
			-- return True if one of our choices is required
		local
			cursor: DS_LINEAR_CURSOR [CHILD_DEFINITION]
		do
			debug
				print ("CHILD_DEFINITION_CHOICE.any_required_child.%N")
			end

			-- if we have been set, we can just return False.
			if state.child_already_set (Current) then
				Result := False
			else
				-- else check if one of our children is required
				from
					cursor := choices.new_cursor
					cursor.start
				until
					Result or else cursor.after
				loop
					Result := cursor.item.any_required_child (state)
					cursor.forth
				end
			end
		end


invariant

	choices_not_void: choices /= Void
	at_least_one_choice: not choices.is_empty

end
