indexing

	description: "Describes a single element in an <!ATTLIST> definition."

	author:     "Berend de Boer <berend@pobox.com>"
	copyright:  "Copyright (c) 2002 by Berend de Boer, see forum.txt"
	date:       "$Date: $"
	revision:   "$Revision: $"


class

	DEFINED_ATTRIBUTE


inherit

	HASHABLE

	EXCEPTIONS
		export {NONE} all end


creation

	make,
	make_ns


feature {NONE} -- Initialization

	make (a_name: STRING; a_required: BOOLEAN; re_content: STRING) is
			-- Initialize attribute. If re_content is not Void, it should
			-- be a valid regular expression.
		require
			name_not_empty: a_name /= Void and then not a_name.is_empty
		do
			name := a_name
			is_required := a_required
			if re_content /= Void then
				create {RX_PCRE_REGULAR_EXPRESSION} allowed_content.make
				allowed_content.compile (re_content)
				if not allowed_content.is_compiled then
					raise ("Regular expression " + re_content + " did not compile.")
				end
			end
		end

	make_ns (a_namespace, a_name: STRING; a_required: BOOLEAN; re_content: STRING) is
			-- Initialize attribute. If re_content is not Void, it should
			-- be a valid regular expression.
		require
			name_not_empty: a_name /= Void and then not a_name.is_empty
		local
			full_name: STRING
		do
			if a_namespace = Void or else a_namespace.is_empty then
				full_name := a_name
			else
				create full_name.make (a_namespace.count + 1 + a_name.count)
				full_name.append_string (a_namespace)
				full_name.append_character (':')
				full_name.append_string (a_name)
			end
			make (full_name, a_required, re_content)
		end


feature -- Access

	allowed_content: RX_REGULAR_EXPRESSION
			-- If not void, it defines the allowed content for this attribute.

	name: STRING


feature -- Callback support

	on_content: PROCEDURE [ANY, TUPLE]
			-- A client can set this to be able to immediately call the
			-- procedure which handles this attributes.

	set_on_content (a_value: PROCEDURE [ANY, TUPLE]) is
			-- Set `on_content'.
		do
			on_content := a_value
		ensure
			on_content_set: on_content = a_value
		end


feature -- Checks

	is_required: BOOLEAN
			-- True if this attribute must occur.

	is_valid_content (a_value: STRING): BOOLEAN is
			-- Is this content valid for this attribute?
		do
			if allowed_content = Void then
				Result := True
			else
				allowed_content.match (a_value)
				Result := allowed_content.has_matched
			end
		ensure
			no_regular_expression_is_true: allowed_content = Void implies Result
		end


feature -- Hashing

	hash_code: INTEGER is
			-- The hash-code value of `Current'.
		do
			Result := name.hash_code
		end


invariant

	name_not_empty: name /= Void and then not name.is_empty
	compiled_regular_expression: allowed_content /= Void implies allowed_content.is_compiled

end
