indexing

	description: "Describes element type that only allows #PCDATA."


class

	DEFINED_ELEMENT_TEXT


inherit

	DEFINED_ELEMENT
		rename
			make as defined_element_make,
			make_ns as defined_element_make_ns
		end

	EXCEPTIONS
		export {NONE} all end


creation

	make,
	make_ns


feature {NONE} -- Initialization

	make (a_name: STRING; re_content: STRING) is
			-- Initialize element with content. If re_content is not
			-- Void, it should be a valid regular expression.
		require
			name_not_empty: a_name /= Void and then not a_name.is_empty
		do
			defined_element_make (a_name)
			set_allowed_content (re_content)
		end

	make_ns (a_namespace, a_name: STRING; re_content: STRING) is
			-- Create element in optional name space.
		require
			name_not_empty: a_name /= Void and then not a_name.is_empty
		do
			defined_element_make_ns (a_namespace, a_name)
			set_allowed_content (re_content)
		end


feature -- Queries

	is_valid_child (state: ELEMENT_STATE;
						 child: DEFINED_ELEMENT): BOOLEAN is
		do
			-- no children allowed
			Result := False
		end

	is_valid_end (state: ELEMENT_STATE): BOOLEAN is
		do
			Result := True
		end

	is_data_allowed: BOOLEAN is
		do
			-- no data allowed
			Result := True
		end

	is_valid_data (data: STRING): BOOLEAN is
			-- Match data against `allowed_content', if set.
		do
			if allowed_content = Void then
				Result := True
			else
				allowed_content.match (data)
				Result := allowed_content.has_matched
			end
		end


feature -- Content

	allowed_content: RX_REGULAR_EXPRESSION
			-- If not void, it defines the allowed content for this attribute.


feature {NONE} --  Implementation

	set_allowed_content (re_content: STRING) is
		do
			if re_content /= Void then
				create {RX_PCRE_REGULAR_EXPRESSION} allowed_content.make
				allowed_content.compile (re_content)
				if not allowed_content.is_compiled then
					raise ("Regular expression " + re_content + " did not compile.")
				end
			else
				allowed_content := Void
			end
		end


invariant

	compiled_regular_expression: allowed_content /= Void implies allowed_content.is_compiled

end
