indexing

	description: "State unique for a single element."


class

	ELEMENT_STATE


creation

	make


feature {NONE} -- Initialization

	make (a_element: DEFINED_ELEMENT) is
		require
			element_not_void: a_element /= Void
		do
			element := a_element
			-- Important: set seen_attributes first, else class invariant violation (when will we see compilers that implement the spec)...
			create seen_attributes.make (element.attributes.count * 3)
			element.init_element_state (Current)
		end


feature -- State

	element: DEFINED_ELEMENT
			-- What element we're tracking state for.


feature -- Report errors back up to highest level

	expected_element: DEFINED_ELEMENT
			-- When an error occurs, it indicates the element that should
			-- have been present.

	set_expected_element (a_element: DEFINED_ELEMENT) is
			-- Set the element that is expected.
		do
			expected_element := a_element
		ensure
			expected_element_set: expected_element = a_element
		end


feature -- Track state for attributes

	seen_attributes: DS_HASH_TABLE [BOOLEAN, DEFINED_ATTRIBUTE]

	add_seen_attribute (attribute: DEFINED_ATTRIBUTE) is
			-- Register the fact that we've seen a certain attribute.
		do
			seen_attributes.put (True, attribute)
		ensure
			seen: seen_attributes.has (attribute)
		end


feature -- Walking child definitions

	children: DS_LINKED_STACK [CHILD_STATE]

	current_child_state: CHILD_STATE is
		require
			at_least_one_child: children /= Void and then not children.is_empty
		do
			Result := children.item
		ensure
			valid_result: Result /= Void
		end

	current_child: CHILD_DEFINITION is
		do
			Result := current_child_state.child
		ensure
			valid_result: Result /= Void
		end


feature {DEFINED_ELEMENT} -- Create state

	create_children is
			-- Called by DEFINED_ELEMENT when needed.
		do
			create children.make
		ensure
			has_children: children /= Void
		end


feature {CHILD_DEFINITION, DEFINED_ELEMENT} -- State used by children validation

	active_child: CHILD_DEFINITION is
		require
			stack_not_empty: not children.is_empty
		do
			Result := children.item.child
		end

	child_already_set (a_child: CHILD_DEFINITION): BOOLEAN is
			-- Returns True if `a_child' already has been set, aka is current.
			-- To be used in `is_allowed_stay' and `any_required_child'.
		require
			valid_child: a_child /= Void
		do
			if children.is_empty then
				Result := False
			else
				Result := active_child = a_child
			end
		end

	authoritative_false: BOOLEAN
			-- If tag is not valid according to some DEFINED_ELEMENT,
			-- does it mean that it is wrong, or simply that a match
			-- should be found elsewhere (a level up).

	last_element_matched: BOOLEAN
			-- Used to check code correctness.

	set_last_element_matched is
		do
			last_element_matched := True
		end


feature {CHILD_DEFINITION, DEFINED_ELEMENT} -- State setting

	set_authoritative_false (value: BOOLEAN) is
			--- Do we really now the final answer?
		do
			authoritative_false := value
		end


invariant

	element_not_void: element /= Void
	seen_attributes_not_void: seen_attributes /= Void

end
