indexing

	description: "Test validation of attributes."


deferred class

	TEST_ATTRIBUTES


inherit

	TS_TEST_CASE

	GF_FORMAT


feature {NONE} -- State

	validator: XM_VALIDATING_PARSER is
			-- Our validating parser, based upon Expat.
		local
			xp: XM_EXPAT_PARSER
			v: XM_VALIDATING_PARSER
		once
			create xp.make
			create v.make (xp)
			Result := v
		end


feature -- The big test

	test_all is
		do
			test1
			test1_bad1

			test2
			test2_bad1
			test2_bad2

			test3
			test3_bad1
			test3_bad2

			test4
			test4_bad1
			test4_bad2
		end

	do_parse (filename: STRING; expected_result: BOOLEAN; assert_msg: STRING) is
		require
			filename_not_empty: filename /= Void and then not filename.is_empty
			assert_msg_not_empty: not expected_result implies (assert_msg /= Void and then not assert_msg.is_empty)
		local
			file: KL_TEXT_INPUT_FILE
		do
			debug ("XML")
				print ("Parsing: ")
				print (filename)
				print ("%N")
			end
			create file.make (filename)
			file.open_read
				check
					file_open: file.is_open_read
				end
			validator.parse_from_stream (file)
			debug ("XML")
				if not validator.is_correct then
					print ("Some error: ")
					print (validator.last_error_extended_description)
					print ("%N")
					print ("Position:")
					print (validator.last_error_position.out)
					print ("%N")
				end
			end
			if expected_result then
				assert (format("Expected parsing of ?s ok:", <<filename>>), validator.is_correct)
			else
				assert (assert_msg, not validator.is_correct)
			end
		end


feature -- test1, no attributes

	test1 is
			-- Most basic test, just check if things are working.
		local
			root: DEFINED_ELEMENT_EMPTY
		do
			validator.wipe_out_schema

			-- build this DTD:
			-- <!ELEMENT root EMPTY>
			create root.make ("root")
			validator.add_element (root)

			do_parse ("test1_ok1.xml", True, Void)
		end

	test1_bad1 is
			-- As `test1', but pass attribute.
		do
			do_parse ("test1_bad1.xml", False, "Should complain about invalid attribute.")
		end


feature -- test2, single optional attribute

	test2 is
			-- Just one attribute.
		local
			root: DEFINED_ELEMENT_EMPTY
			attribute: DEFINED_ATTRIBUTE
		do
			validator.wipe_out_schema

			-- build this DTD:
			-- <!ELEMENT root EMPTY>
			-- <!ATTLIST root version CDATA #IMPLIED>
			create root.make ("root")
			create attribute.make ("version", False, Void)
			root.add_attribute (attribute)
			validator.add_element (root)

			do_parse ("test2_ok1.xml", True, Void)
			do_parse ("test2_ok2.xml", True, Void)
		end

	test2_bad1 is
			-- As `test2', but pass an additional attribute.
		do
			do_parse ("test2_bad1.xml", False, "Should complain about invalid attribute.")
		end

	test2_bad2 is
			-- As `test2', but pass an additional attribute.
		do
			do_parse ("test2_bad2.xml", False, "Should complain about invalid attribute.")
		end


feature -- test3, single required attribute

	test3 is
			-- Just one attribute.
		local
			root: DEFINED_ELEMENT_EMPTY
			attribute: DEFINED_ATTRIBUTE
		do
			validator.wipe_out_schema

			-- build this DTD:
			-- <!ELEMENT root EMPTY>
			-- <!ATTLIST root version CDATA #REQUIRED>
			create root.make ("root")
			create attribute.make ("version", True, Void)
			root.add_attribute (attribute)
			validator.add_element (root)

			do_parse ("test3_ok1.xml", True, Void)
		end

	test3_bad1 is
			-- As `test3', but no attribute of course.
		do
			do_parse ("test3_bad1.xml", False, "Should complain about missing attribute version.")
		end

	test3_bad2 is
			-- As `test2', but pass the wrong attribute.
		do
			do_parse ("test3_bad2.xml", False, "Should complain about invalid attribute.")
		end


feature -- test4, test attribute contents

	test4 is
			-- Just one attribute.
		local
			root: DEFINED_ELEMENT_EMPTY
			attribute: DEFINED_ATTRIBUTE
		do
			validator.wipe_out_schema

			-- build this DTD:
			-- <!ELEMENT root EMPTY>
			-- <!ATTLIST root version CDATA #REQUIRED>
			create root.make ("root")
			create attribute.make ("version", True, "[0-9]+\.[0-9]+")
			root.add_attribute (attribute)
			validator.add_element (root)

			do_parse ("test4_ok1.xml", True, Void)
		end

	test4_bad1 is
			-- As `test4', but wrong content.
		do
			do_parse ("test4_bad1.xml", False, "Should complain about wrong content for attribute version.")
		end

	test4_bad2 is
			-- As `test4', but empty content.
		do
			do_parse ("test4_bad2.xml", False, "Should complain about wrong content for attribute version.")
		end


end
