indexing

	description: "Basic tests that validation works."


deferred class

   TEST_VALIDATOR


inherit

	TS_TEST_CASE

	GF_FORMAT


feature {NONE} -- State

   validator: XM_VALIDATING_PARSER is
			-- Our validating parser, based upon Expat.
		local
			xp: XM_EXPAT_PARSER
			v: XM_VALIDATING_PARSER
      once
			create xp.make
			create v.make (xp)
			Result := v
      end


feature -- The big test

   test_all is
      do
         test1
         test1_bad1
         test1_bad2
         test1_bad3

         test2
         test2_bad1
         test2_bad2

         test3
         test3_bad1

         test4
         test4_bad1
         test4_bad2

         test5
         test5_bad1
         test5_bad2
         test5_bad3
         test5_bad4

         test6

         test7
         test7_bad1

         test8
         test8_bad1

         test9
         test9_bad1

         test10
         test10_bad1
         test10_bad2
         test10_bad3

         test11
         test11_bad1
         test11_bad2

         test12
         test12_bad1

         test13
      end

   do_parse (filename: STRING; expected_result: BOOLEAN; assert_msg: STRING) is
		require
			filename_not_empty: filename /= Void and then not filename.is_empty
			assert_msg_not_empty: not expected_result implies (assert_msg /= Void and then not assert_msg.is_empty)
		local
			file: KL_TEXT_INPUT_FILE
      do
			debug ("XML")
				print ("Parsing: ")
				print (filename)
				print ("%N")
			end
			create file.make (filename)
			file.open_read
				check
					file_open: file.is_open_read
				end
         validator.parse_from_stream (file)
			if expected_result then
				assert (format("Expected parsing of ?s ok:", <<filename>>), validator.is_correct)
			else
				debug ("XML")
					if not validator.is_correct then
						print ("Some error: ")
						print (validator.last_error_extended_description)
						print ("%N")
					end
				end
				assert (assert_msg, not validator.is_correct)
			end
      end


feature -- test1, most basic xml

   test1 is
         -- Most basic test, just check if things are working.
      local
         root: DEFINED_ELEMENT_EMPTY
      do
         validator.wipe_out_schema

         -- build this DTD:
         -- <!ELEMENT root EMPTY>
         create root.make ("root")
         validator.add_element (root)

         do_parse ("test1_ok1.xml", True, Void)
         do_parse ("test1_ok2.xml", True, Void)
      end

   test1_bad1 is
         -- As `test1', but pass not allowed char data.
      do
			do_parse ("test1_bad1.xml", False, "Should complain about character data.")
      end

   test1_bad2 is
         -- As `test1', but have invalid end tag.
         -- Should be catched by parser.
      do
         do_parse ("test1_bad2.xml", False, "XML rejected. Should complain about mismatched tag.")
      end

   test1_bad3 is
         -- Invalid doctype tag.
      local
         test: DEFINED_ELEMENT_EMPTY
      do
         create test.make ("test")
         validator.add_element (test)
			do_parse ("test1_bad3.xml", False, "Should complain about wrong document root.")
      end


feature -- test2, somewhat more advanced xml

   test2 is
         -- Elements with children.
      local
         root: DEFINED_ELEMENT_CHILDREN
         a: DEFINED_ELEMENT_EMPTY
         child_def_a: CHILD_DEFINITION_NAME
         child_def_root: CHILD_DEFINITION_SEQ
         list: DS_LINKED_LIST [CHILD_DEFINITION]
      do
         validator.wipe_out_schema

         -- build this DTD:
         -- <!ELEMENT root (a)>
         -- <!ELEMENT a EMPTY>

         create a.make ("a")
         validator.add_element (a)

         create child_def_a.make (a)
         create list.make
         list.put_first (child_def_a)
         create child_def_root.make (list)
         create root.make ("root", child_def_root)
         validator.add_element (root)

         do_parse ("test2_ok1.xml", True, Void)
      end

   test2_bad1 is
         -- as `test2', but pass extra, invalid <a>
      do
			do_parse ("test2_bad1.xml", False, "Exception caught. Should not expect <a>.")
      end

   test2_bad2 is
         -- Just <root/>
      do
			do_parse ("test2_bad2.xml", False, "Exception caught. Should expect <a>.")
      end


feature -- test3, even more advanced xml

   test3 is
         -- Elements with children.
      local
         root: DEFINED_ELEMENT_CHILDREN
         a,
         b: DEFINED_ELEMENT_EMPTY
         child_def_a,
         child_def_b: CHILD_DEFINITION_NAME
         child_def_root: CHILD_DEFINITION_SEQ
         list: DS_LINKED_LIST [CHILD_DEFINITION]
      do
         validator.wipe_out_schema

         -- build this DTD:
         -- <!ELEMENT root (a,b)>
         -- <!ELEMENT a EMPTY>
         -- <!ELEMENT b EMPTY>

         create a.make ("a")
         validator.add_element (a)

         create b.make ("b")
         validator.add_element (b)

         create child_def_a.make (a)
         create child_def_b.make (b)
         create list.make
         list.put_first (child_def_a)
         list.put_last (child_def_b)
         create child_def_root.make (list)
         create root.make ("root", child_def_root)
         validator.add_element (root)

         do_parse ("test3_ok1.xml", True, Void)
      end

   test3_bad1 is
         -- should reject test2_ok1.xml, because tag b is required
      do
			do_parse ("test3_bad1.xml", False, "Should complain that tag <b> expected.")
      end


feature -- test4, DTD with choice definition

   test4 is
         -- Elements with choices.
      local
         root: DEFINED_ELEMENT_CHILDREN
         a,
         b: DEFINED_ELEMENT_EMPTY
         child_def_a,
         child_def_b: CHILD_DEFINITION_NAME
         child_def_root: CHILD_DEFINITION_CHOICE
         choices: DS_LINKED_LIST [CHILD_DEFINITION]
      do
         validator.wipe_out_schema

         -- build this DTD:
         -- <!ELEMENT root (a|b)>
         -- <!ELEMENT a EMPTY>
         -- <!ELEMENT b EMPTY>

         create a.make ("a")
         validator.add_element (a)

         create b.make ("b")
         validator.add_element (b)

         create child_def_a.make (a)
         create child_def_b.make (b)
         create choices.make
         choices.put_first (child_def_a)
         choices.put_last (child_def_b)
         create child_def_root.make (choices)
         create root.make ("root", child_def_root)
         validator.add_element (root)

         do_parse ("test4_ok1.xml", True, Void)
      end

   test4_bad1 is
         -- Should reject b tag because a tag was already matched.
      do
			do_parse ("test4_bad1.xml", False, "Should reject <b>.")
      end

   test4_bad2 is
         -- No child tags, should detect that it needs at least one.
      do
			do_parse ("test4_bad2.xml", False, "Need at least one child tag.")
      end


feature -- test5, DTD with choice and list definition

   test5 is
         -- Elements with choices.
      local
         root: DEFINED_ELEMENT_CHILDREN
         a,
         b: DEFINED_ELEMENT_EMPTY
         child_def_a,
         child_def_b: CHILD_DEFINITION_NAME
         child_def_root: CHILD_DEFINITION_CHOICE
         child_def_seq: CHILD_DEFINITION_SEQ
         list: DS_LINKED_LIST [CHILD_DEFINITION]
         choices: DS_LINKED_LIST [CHILD_DEFINITION]
      do
         validator.wipe_out_schema

         -- build this DTD:
         -- <!ELEMENT root ((a,b) | b)>
         -- <!ELEMENT a EMPTY>
         -- <!ELEMENT b EMPTY>

         create a.make ("a")
         validator.add_element (a)

         create b.make ("b")
         validator.add_element (b)

         create child_def_a.make (a)
         create child_def_b.make (b)
         create list.make
         list.put_first (child_def_a)
         list.put_last (child_def_b)
         create child_def_seq.make (list)

         create child_def_b.make (b)
         create choices.make
         choices.put_first (child_def_seq)
         choices.put_last (child_def_b)
         create child_def_root.make (choices)
         create root.make ("root", child_def_root)
         validator.add_element (root)

         do_parse ("test5_ok1.xml", True, Void)
         do_parse ("test5_ok2.xml", True, Void)
      end

   test5_bad1 is
         -- Only an <a>.
      do
			do_parse ("test5_bad1.xml", False, "Should complain that <b> was expected.")
      end

   test5_bad2 is
         -- <a/><b/><b/>.
      do
			do_parse ("test5_bad2.xml", False, "Second <b> rejected.")
      end

   test5_bad3 is
         -- <a/><b/><a/>
      do
			do_parse ("test5_bad3.xml", False, "Second <a> rejected.")
      end

   test5_bad4 is
         -- only <root>.
      do
			do_parse ("test5_bad4.xml", False, "Need either a <a> or a <b>.")
      end


feature -- test6, occurrence *

   test6 is
         -- Repeating elements.
      local
         root: DEFINED_ELEMENT_CHILDREN
         a: DEFINED_ELEMENT_EMPTY
         child_def_a: CHILD_DEFINITION_NAME
         child_def_occurrence: CHILD_DEFINITION_OCCURRENCE
         child_def_root: CHILD_DEFINITION_CHOICE
         list: DS_LINKED_LIST [CHILD_DEFINITION]
      do
         validator.wipe_out_schema

         -- build this DTD:
         -- <!ELEMENT root (a*)>
         -- <!ELEMENT a EMPTY>

         create a.make ("a")
         validator.add_element (a)

         create child_def_a.make (a)
         create child_def_occurrence.make (child_def_a, 0, Maximum_integer)
         create list.make
         list.put_first (child_def_occurrence)
         create child_def_root.make (list)
         create root.make ("root", child_def_root)
         validator.add_element (root)

         do_parse ("test6_ok1.xml", True, Void)
         do_parse ("test6_ok2.xml", True, Void)
         do_parse ("test6_ok3.xml", True, Void)
      end


feature -- test7, occurrence +

   test7 is
         -- Repeating elements.
      local
         root: DEFINED_ELEMENT_CHILDREN
         a: DEFINED_ELEMENT_EMPTY
         child_def_a: CHILD_DEFINITION_NAME
         child_def_occurrence: CHILD_DEFINITION_OCCURRENCE
         child_def_root: CHILD_DEFINITION_CHOICE
         list: DS_LINKED_LIST [CHILD_DEFINITION]
      do
         validator.wipe_out_schema

         -- build this DTD:
         -- <!ELEMENT root (a+)>
         -- <!ELEMENT a EMPTY>

         create a.make ("a")
         validator.add_element (a)

         create child_def_a.make (a)
         create child_def_occurrence.make (child_def_a, 1, Maximum_integer)
         create list.make
         list.put_first (child_def_occurrence)
         create child_def_root.make (list)
         create root.make ("root", child_def_root)
         validator.add_element (root)

         do_parse ("test7_ok1.xml", True, Void)
         do_parse ("test7_ok2.xml", True, Void)
      end

   test7_bad1 is
         -- Missing <a>.
      do
			do_parse ("test7_bad1.xml", False, "Complain about missing <a>.")
      end


feature -- test8, occurrence ?

   test8 is
         -- Repeating elements.
      local
         root: DEFINED_ELEMENT_CHILDREN
         a: DEFINED_ELEMENT_EMPTY
         child_def_a: CHILD_DEFINITION_NAME
         child_def_occurrence: CHILD_DEFINITION_OCCURRENCE
         child_def_root: CHILD_DEFINITION_CHOICE
         list: DS_LINKED_LIST [CHILD_DEFINITION]
      do
         validator.wipe_out_schema

         -- build this DTD:
         -- <!ELEMENT root (a?)>
         -- <!ELEMENT a EMPTY>

         create a.make ("a")
         validator.add_element (a)

         create child_def_a.make (a)
         create child_def_occurrence.make (child_def_a, 0, 1)
         create list.make
         list.put_first (child_def_occurrence)
         create child_def_root.make (list)
         create root.make ("root", child_def_root)
         validator.add_element (root)

         do_parse ("test8_ok1.xml", True, Void)
         do_parse ("test8_ok2.xml", True, Void)
      end

   test8_bad1 is
         -- Extra <a>.
      do
			do_parse ("test8_bad1.xml", False, "Complain about extra <a>.")
      end


feature -- test9, More nesting

   test9 is
         -- Multiple nesting.
      local
         root: DEFINED_ELEMENT_CHILDREN
         a,
         b: DEFINED_ELEMENT_EMPTY
         c: DEFINED_ELEMENT_CHILDREN
         child_def_root_a,
         child_def_root_b: CHILD_DEFINITION_NAME
         child_def_root_c: CHILD_DEFINITION_NAME
         child_def_root: CHILD_DEFINITION_CHOICE
         child_def_a,
         child_def_b: CHILD_DEFINITION_NAME
         child_def_seq1: CHILD_DEFINITION_SEQ
         child_def_seq2: CHILD_DEFINITION_SEQ
         child_def_c: CHILD_DEFINITION_SEQ
         list: DS_LINKED_LIST [CHILD_DEFINITION]
         choices: DS_LINKED_LIST [CHILD_DEFINITION]
      do
         validator.wipe_out_schema

         -- build this DTD:
         --  <!ELEMENT root (a | b | c)>
         --  <!ELEMENT a EMPTY>
         --  <!ELEMENT b EMPTY>
         --  <!ELEMENT c ((a, b), (a, b), b)>

         create a.make ("a")
         validator.add_element (a)

         create b.make ("b")
         validator.add_element (b)

         create child_def_a.make (a)
         create child_def_b.make (b)
         create list.make
         list.put_last (child_def_a)
         list.put_last (child_def_b)
         create child_def_seq1.make (list)
         create child_def_a.make (a)
         create child_def_b.make (b)
         create list.make
         list.put_last (child_def_a)
         list.put_last (child_def_b)
         create child_def_seq2.make (list)
         create child_def_b.make (b)
         create list.make
         list.put_last (child_def_seq1)
         list.put_last (child_def_seq2)
         list.put_last (child_def_b)
         create child_def_c.make (list)
         create c.make ("c", child_def_c)
         validator.add_element (c)

         create child_def_root_a.make (a)
         create child_def_root_b.make (b)
         create child_def_root_c.make (c)
         create choices.make
         choices.put_last (child_def_root_a)
         choices.put_last (child_def_root_b)
         choices.put_last (child_def_root_c)
         create child_def_root.make (choices)
         create root.make ("root", child_def_root)
         validator.add_element (root)

         do_parse ("test9_ok1.xml", True, Void)
         do_parse ("test9_ok2.xml", True, Void)
      end

   test9_bad1 is
         -- Wrong inside <c>: <c><a/><a/></c>.
      do
			do_parse ("test9_bad1.xml", False, "Should expect <b>.")
      end


feature -- test10, more nesting

   test10 is
         -- Multiple nesting.
      local
         root: DEFINED_ELEMENT_CHILDREN
         a,
         b: DEFINED_ELEMENT_EMPTY
         c: DEFINED_ELEMENT_CHILDREN
         child_def_root_a,
         child_def_root_b: CHILD_DEFINITION_NAME
         child_def_root_c: CHILD_DEFINITION_NAME
         child_def_root: CHILD_DEFINITION_CHOICE
         child_def_a,
         child_def_b: CHILD_DEFINITION_NAME
         child_def_choice1: CHILD_DEFINITION_CHOICE
         child_def_seq2: CHILD_DEFINITION_SEQ
         child_def_c: CHILD_DEFINITION_SEQ
         list: DS_LINKED_LIST [CHILD_DEFINITION]
         choices: DS_LINKED_LIST [CHILD_DEFINITION]
      do
         validator.wipe_out_schema

         -- build this DTD:
         --  <!ELEMENT root (a | b | c)>
         --  <!ELEMENT a EMPTY>
         --  <!ELEMENT b EMPTY>
         --  <!ELEMENT c ((a | b), (a, b), b)>

         create a.make ("a")
         validator.add_element (a)

         create b.make ("b")
         validator.add_element (b)

         create child_def_a.make (a)
         create child_def_b.make (b)
         create choices.make
         choices.put_last (child_def_a)
         choices.put_last (child_def_b)
         create child_def_choice1.make (choices)
         create child_def_a.make (a)
         create child_def_b.make (b)
         create list.make
         list.put_last (child_def_a)
         list.put_last (child_def_b)
         create child_def_seq2.make (list)
         create child_def_b.make (b)
         create list.make
         list.put_last (child_def_choice1)
         list.put_last (child_def_seq2)
         list.put_last (child_def_b)
         create child_def_c.make (list)
         create c.make ("c", child_def_c)
         validator.add_element (c)

         create child_def_root_a.make (a)
         create child_def_root_b.make (b)
         create child_def_root_c.make (c)
         create choices.make
         choices.put_last (child_def_root_a)
         choices.put_last (child_def_root_b)
         choices.put_last (child_def_root_c)
         create child_def_root.make (choices)
         create root.make ("root", child_def_root)
         validator.add_element (root)

         do_parse ("test10_ok1.xml", True, Void)
         do_parse ("test10_ok2.xml", True, Void)
      end

   test10_bad1 is
         -- Wrong inside <c>: <c><a/><a/></c>.
      do
			do_parse ("test10_bad1.xml", False, "Should expect <b>.")
      end

   test10_bad2 is
         -- </c> too soon: <root><c><a/></c></root>.
      do
			do_parse ("test10_bad2.xml", False, "Should expect <a>.")
      end

   test10_bad3 is
         -- </c> too soon: <root><c><a/><a/><b/></c></root>.
      do
			do_parse ("test10_bad3.xml", False, "Should expect <b>.")
      end


feature -- test11, sequence and occurrence ?

   test11 is
         -- Repeating elements.
      local
         root: DEFINED_ELEMENT_CHILDREN
         a: DEFINED_ELEMENT_EMPTY
         b: DEFINED_ELEMENT_EMPTY
         c: DEFINED_ELEMENT_EMPTY
         child_def_a: CHILD_DEFINITION_NAME
         child_def_b: CHILD_DEFINITION_NAME
         child_def_occurrence: CHILD_DEFINITION_OCCURRENCE
         child_def_root: CHILD_DEFINITION_SEQ
         list: DS_LINKED_LIST [CHILD_DEFINITION]
      do
         validator.wipe_out_schema

         -- build this DTD:
         -- <!ELEMENT root (a?, b)>
         -- <!ELEMENT a EMPTY>
         -- <!ELEMENT b EMPTY>
         -- <!ELEMENT c EMPTY>

         create a.make ("a")
         validator.add_element (a)

         create b.make ("b")
         validator.add_element (b)

         create c.make ("c")
         validator.add_element (c)

         create child_def_a.make (a)
         create child_def_occurrence.make (child_def_a, 0, 1)
         create child_def_b.make (b)
         create list.make
         list.put_first (child_def_occurrence)
         list.put_last (child_def_b)
         create child_def_root.make (list)
         create root.make ("root", child_def_root)
         validator.add_element (root)

         do_parse ("test11_ok1.xml", True, Void)
         do_parse ("test11_ok2.xml", True, Void)
      end

   test11_bad1 is
         -- huu? <root><c></root>.
      do
			do_parse ("test11_bad1.xml", False, "Should expect <b>.")
      end

   test11_bad2 is
         -- <root><a/></a></root>.
      do
			do_parse ("test11_bad2.xml", False, "Should expect <b>.")
      end


feature -- test12, sequence and occurrence ?

   test12 is
         -- Repeating elements
      local
         root: DEFINED_ELEMENT_CHILDREN
         a: DEFINED_ELEMENT_EMPTY
         b: DEFINED_ELEMENT_EMPTY
         c: DEFINED_ELEMENT_EMPTY
         child_def_a: CHILD_DEFINITION_NAME
         child_def_b: CHILD_DEFINITION_NAME
         child_def_c: CHILD_DEFINITION_NAME
         child_def_occurrence: CHILD_DEFINITION_OCCURRENCE
         child_def_root: CHILD_DEFINITION_SEQ
         list: DS_LINKED_LIST [CHILD_DEFINITION]
      do
         validator.wipe_out_schema

         -- build this DTD:
         -- <!ELEMENT root (a, b?, c)>
         -- <!ELEMENT a EMPTY>
         -- <!ELEMENT b EMPTY>
         -- <!ELEMENT c EMPTY>

         create a.make ("a")
         validator.add_element (a)

         create b.make ("b")
         validator.add_element (b)

         create c.make ("c")
         validator.add_element (c)

         create child_def_a.make (a)
         create child_def_b.make (b)
         create child_def_occurrence.make (child_def_b, 0, 1)
         create child_def_c.make (c)
         create list.make
         list.put_first (child_def_a)
         list.put_last (child_def_occurrence)
         list.put_last (child_def_c)
         create child_def_root.make (list)
         create root.make ("root", child_def_root)
         validator.add_element (root)

         do_parse ("test12_ok1.xml", True, Void)
         do_parse ("test12_ok2.xml", True, Void)
      end

   test12_bad1 is
         -- huu? <root><a></b></root>.
      do
			do_parse ("test12_bad1.xml", False, "Should expect <c>.")
      end


feature -- test13, sequence and occurrence *

   test13 is
         -- Repeating elements.
      local
         root: DEFINED_ELEMENT_CHILDREN
         a: DEFINED_ELEMENT_EMPTY
         b: DEFINED_ELEMENT_EMPTY
         c: DEFINED_ELEMENT_EMPTY
         child_def_a: CHILD_DEFINITION_NAME
         child_def_b: CHILD_DEFINITION_NAME
         child_def_c: CHILD_DEFINITION_NAME
         child_def_occurrence: CHILD_DEFINITION_OCCURRENCE
         child_def_root: CHILD_DEFINITION_SEQ
         list: DS_LINKED_LIST [CHILD_DEFINITION]
      do
         validator.wipe_out_schema

         -- build this DTD:
         -- <!ELEMENT root (a, b*, c)>
         -- <!ELEMENT a EMPTY>
         -- <!ELEMENT b EMPTY>
         -- <!ELEMENT c EMPTY>

         create a.make ("a")
         validator.add_element (a)

         create b.make ("b")
         validator.add_element (b)

         create c.make ("c")
         validator.add_element (c)

         create child_def_a.make (a)
         create child_def_b.make (b)
         create child_def_occurrence.make (child_def_b, 0, Maximum_integer)
         create child_def_c.make (c)
         create list.make
         list.put_first (child_def_a)
         list.put_last (child_def_occurrence)
         list.put_last (child_def_c)
         create child_def_root.make (list)
         create root.make ("root", child_def_root)
         validator.add_element (root)

         do_parse ("test13_ok1.xml", True, Void)
         do_parse ("test13_ok2.xml", True, Void)
         do_parse ("test13_ok3.xml", True, Void)
      end


end
